__all__ = ('IdIDXPoint',)

from expressmoney_service.api import *

_SERVICE = 'services'


class _IdIDXCreateContract(Contract):
    first_name = serializers.CharField(max_length=32)
    last_name = serializers.CharField(max_length=32)
    middle_name = serializers.CharField(max_length=32)
    birth_date = serializers.DateField()
    passport_serial = serializers.CharField(max_length=4)
    passport_number = serializers.CharField(max_length=6)
    passport_date = serializers.DateField()
    passport_code = serializers.CharField(max_length=16)


class _IdIDXResponseContract(_IdIDXCreateContract):
    pass


class _IdIDXID(ID):
    _service = _SERVICE
    _app = 'id_idx'
    _view_set = 'id_idx'


class IdIDXPoint(ResponseMixin, CreatePointMixin, ContractPoint):
    _point_id = _IdIDXID()
    _create_contract = _IdIDXCreateContract
    _response_contract = _IdIDXResponseContract



from google.cloud import secretmanager
from google.cloud import secretmanager_v1

secret_manager_client = secretmanager.SecretManagerServiceClient()
access_secret_version = secretmanager_v1.types.service.AccessSecretVersionRequest()


def get_secret(secret_key: str):
    name = f'projects/829013617684/secrets/{secret_key}/versions/1'
    access_secret_version.name = name
    return secret_manager_client.access_secret_version(request=access_secret_version).payload.data.decode("utf-8")



__all__ = ('Tele2Point',)

from phonenumber_field.serializerfields import PhoneNumberField

from expressmoney_service.api import *

_SERVICE = 'services'
_APP = 'tele2'


class _Tele2CreateContract(Contract):
    username = PhoneNumberField()
    first_name = serializers.CharField(max_length=32)
    last_name = serializers.CharField(max_length=32)
    middle_name = serializers.CharField(max_length=32, allow_blank=True)


class _Tele2ID(ID):
    _service = _SERVICE
    _app = _APP
    _view_set = 'tele2'


class Tele2Point(CreatePointMixin, ContractPoint):
    _point_id = _Tele2ID()
    _create_contract = _Tele2CreateContract



__all__ = ('EfinILOrderPoint', 'EfinCancelOrderPoint')

from ..api import *
from phonenumber_field.serializerfields import PhoneNumberField

_SERVICE = 'services'


class _EfinILOrderCreateContract(Contract):
    product_id = serializers.IntegerField()
    first_name = serializers.CharField(max_length=32)
    last_name = serializers.CharField(max_length=32)
    middle_name = serializers.CharField(max_length=32)
    birth_date = serializers.DateField()
    city = serializers.CharField(max_length=32)
    address = serializers.CharField(max_length=250)
    phonenumber = PhoneNumberField()
    passport_serial = serializers.CharField(max_length=4)
    passport_number = serializers.CharField(max_length=6)
    passport_code = serializers.CharField(max_length=16)
    passport_date = serializers.DateField()
    meeting_day = serializers.DateField()


class _EfinILOrderReadContract(Contract):
    created = serializers.DateTimeField()
    meeting_day = serializers.DateField()
    sign_day = serializers.DateField()
    status_code = serializers.IntegerField()
    status_name = serializers.CharField(max_length=150)
    status_description = serializers.CharField(max_length=256)
    status_date = serializers.DateTimeField(allow_null=True)


class _EfinCancelOrderCreateContract(Contract):
    pass


class _EfinILOrder(ID):
    _service = _SERVICE
    _app = 'efin'
    _view_set = 'efin_il_order'


class _EfinCancelOrder(ID):
    _service = _SERVICE
    _app = 'efin'
    _view_set = 'set_canceled'


class EfinILOrderPoint(CreatePointMixin, ListPointMixin, ContractPoint):
    _point_id = _EfinILOrder()
    _create_contract = _EfinILOrderCreateContract
    _read_contract = _EfinILOrderReadContract
    _sort_by = 'created'


class EfinCancelOrderPoint(CreatePointMixin, ContractPoint):
    _point_id = _EfinCancelOrder()
    _create_contract = _EfinCancelOrderCreateContract



__all__ = ('SmsMTSPoint', 'SmsMTS2Point', 'SmsToReferralPoint')

from expressmoney_service.api import *

_SERVICE = 'services'


class _SmsMTSCreateContract(Contract):
    message = serializers.CharField(max_length=60)


class _SmsMTS2CreateContract(Contract):
    message = serializers.CharField(max_length=60)


class _SmsToReferralCreateContract(Contract):
    message = serializers.CharField(max_length=130)
    phone_number = serializers.CharField(max_length=20)


class _SmsMTSResponseContract(_SmsMTSCreateContract):
    pass


class _SmsMTSID(ID):
    _service = _SERVICE
    _app = 'sms_mts'
    _view_set = 'sms_mts'


class _SmsMTS2ID(ID):
    _service = _SERVICE
    _app = 'sms_mts'
    _view_set = 'sms_mts2'


class _SmsToReferralID(ID):
    _service = _SERVICE
    _app = 'sms_mts'
    _view_set = 'sms_to_referral'


class SmsMTSPoint(ResponseMixin, CreatePointMixin, ContractPoint):
    _point_id = _SmsMTSID()
    _create_contract = _SmsMTSCreateContract
    _response_contract = _SmsMTSResponseContract


class SmsMTS2Point(CreatePointMixin, ContractPoint):
    _point_id = _SmsMTS2ID()
    _create_contract = _SmsMTS2CreateContract


class SmsToReferralPoint(CreatePointMixin, ContractPoint):
    _point_id = _SmsToReferralID()
    _create_contract = _SmsToReferralCreateContract



__all__ = ('EsiaPoint',)

from phonenumber_field.serializerfields import PhoneNumberField

from expressmoney_service.api import *

_SERVICE = 'services'
_APP = 'idx_esia'


class _EsiaCreateContract(Contract):
    phonenumber = PhoneNumberField()
    first_name = serializers.CharField(max_length=32)
    last_name = serializers.CharField(max_length=32)
    middle_name = serializers.CharField(max_length=32)
    birth_date = serializers.DateField()
    passport_serial = serializers.CharField(max_length=4)
    passport_number = serializers.CharField(max_length=6)


class _EsiaID(ID):
    _service = _SERVICE
    _app = _APP
    _view_set = 'esia'


class EsiaPoint(CreatePointMixin, ContractPoint):
    _point_id = _EsiaID()
    _create_contract = _EsiaCreateContract



__all__ = (
    'Point', 'ContractPoint',
    'ResponseMixin', 'CreatePointMixin',
    'ID', 'Contract',
    'ListPointMixin',
)

from typing import OrderedDict

from expressmoney.api import PointNotFound404, PointThrottled, PointClientError, PointServerError
from expressmoney.api.point import PointError
from rest_framework import status

from .client import Request
from .contract import Contract
from .filter import FilterMixin
from .id import ID


class Point:
    """Base endpoint handler"""
    _point_id: ID = None

    def __init__(self,
                 user,
                 timeout: tuple = (30, 30)
                 ):
        self._response = None
        self._client = Request(service=self._point_id.service,
                               path=self._path,
                               user=user,
                               timeout=timeout,
                               )

    @property
    def client(self):
        return self._client

    @property
    def _path(self):
        path = self._point_id.path
        return path

    def _post(self, payload: dict):
        self._response = self._client.post(payload=payload)
        self._handle_error(self._response)

    def _get(self, url=None) -> dict:
        self._response = self._client.get(url)
        self._handle_error(self._response)
        data = self._response.json()
        return data

    def _handle_error(self, response):

        if not status.is_success(response.status_code):
            if status.is_client_error(response.status_code):
                if response.status_code == status.HTTP_404_NOT_FOUND:
                    raise PointNotFound404(self._client.url)
                if response.status_code == status.HTTP_429_TOO_MANY_REQUESTS:
                    raise PointThrottled(self._client.url, response.status_code, response.headers.get(
                        'Retry-After'))
                else:
                    raise PointClientError(self._client.url, response.status_code, response.json())
            else:
                raise PointServerError(self._client.url, response.status_code)


class ContractPoint(FilterMixin, Point):
    """Endpoints with validated data by contract"""
    _read_contract = None
    _create_contract = None
    _sort_by = 'id'

    def __init__(self,
                 user,
                 timeout: tuple = (30, 30),
                 ):
        super().__init__(user=user, timeout=timeout)

    def _get_validated_data(self):
        data = self._get()
        contract = self._get_contract(data, True)
        validated_data = contract.validated_data
        return validated_data

    def _get_contract(self, data, is_read: bool) -> Contract:
        contract_class = self._get_contract_class(is_read)
        contract = contract_class(data=data, many=True if is_read else False)
        contract.is_valid(raise_exception=True)
        return contract

    def _get_contract_class(self, is_read: bool):
        return self._read_contract if is_read else self._create_contract

    def _get_sorted_data(self) -> tuple:
        if self._sort_by is None:
            raise PointError('Set key for sort or False')
        validated_data = self._get_validated_data()
        sorted_data = sorted(validated_data, key=lambda obj: obj[self._sort_by]) if self._sort_by else validated_data
        return tuple(sorted_data)


class CreatePointMixin:
    """For type ContractPoint"""

    def create(self, payload: dict):
        if self._create_contract is None:
            raise PointError(f'Set attr create_contract')
        contract = self._get_contract(data=payload, is_read=False)
        self._post(contract.data)


class ResponseMixin:
    """Only for create and update actions"""

    _response_contract = None

    @property
    def response(self) -> OrderedDict:
        if self._response_contract is None:
            raise PointError('Response contract not set')
        if self._response is None:
            raise PointError('First create or update data')
        if self._response.status_code != status.HTTP_201_CREATED:
            raise PointError(f'Response data only for 201 status, current {self._response.status_code}')
        contract = self._response_contract(data=self._response.json())
        contract.is_valid(raise_exception=True)
        return contract.validated_data


class ListPointMixin:
    def list(self) -> tuple:
        if self._read_contract is None:
            raise PointError(f'Set attr read_contract')
        return self._get_sorted_data()



__all__ = ('ContractCreateStatusPoint',)

from expressmoney_service.api import *

_SERVICE = 'services'


class ContractStatusCreateContract(Contract):
    inn = serializers.CharField(max_length=32)


class ContractStatusResponseContract(Contract):
    status = serializers.CharField(max_length=16)


class _ContractUpdateID(ID):
    _service = _SERVICE
    _app = 'contracts'
    _view_set = 'contract_status'


class ContractCreateStatusPoint(ResponseMixin, CreatePointMixin, ContractPoint):
    _point_id = _ContractUpdateID()
    _create_contract = ContractStatusCreateContract
    _response_contract = ContractStatusResponseContract
